package com.fjhx.wms.service.stock.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.entity.product.IssueImportExcelBo;
import com.fjhx.item.entity.product.dto.ProductInfoSelectDto;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.entity.product.vo.ProductInfoVo;
import com.fjhx.item.service.product.ProductClassifyService;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.wms.entity.stock.dto.StockCheckDto;
import com.fjhx.wms.entity.stock.dto.StockCheckSelectDto;
import com.fjhx.wms.entity.stock.dto.StockSelectDto;
import com.fjhx.wms.entity.stock.po.Stock;
import com.fjhx.wms.entity.stock.po.StockCheck;
import com.fjhx.wms.entity.stock.po.StockCheckDetails;
import com.fjhx.wms.entity.stock.po.StockCheckRegister;
import com.fjhx.wms.entity.stock.vo.StockCheckDetailsVo;
import com.fjhx.wms.entity.stock.vo.StockCheckRegisterVo;
import com.fjhx.wms.entity.stock.vo.StockCheckVo;
import com.fjhx.wms.entity.stock.vo.StockVo;
import com.fjhx.wms.mapper.stock.StockCheckMapper;
import com.fjhx.wms.service.stock.StockCheckDetailsService;
import com.fjhx.wms.service.stock.StockCheckRegisterService;
import com.fjhx.wms.service.stock.StockCheckService;
import com.fjhx.wms.service.stock.StockService;
import com.fjhx.wms.service.warehouse.WarehouseService;
import com.fjhx.wms.utils.CodeEnum;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 库存盘点 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-22
 */
@Service
public class StockCheckServiceImpl extends ServiceImpl<StockCheckMapper, StockCheck> implements StockCheckService {

    private final StockCheckDetailsService stockCheckDetailsService;
    private final ProductInfoService productInfoService;
    private final WarehouseService warehouseService;
    private final StockService stockService;
    private final ProductClassifyService productClassifyService;
    private final StockCheckRegisterService stockCheckRegisterService;

    @Autowired
    public StockCheckServiceImpl(StockCheckDetailsService stockCheckDetailsService, ProductInfoService productInfoService, WarehouseService warehouseService, StockService stockService, ProductClassifyService productClassifyService, StockCheckRegisterService stockCheckRegisterService) {
        this.stockCheckDetailsService = stockCheckDetailsService;
        this.productInfoService = productInfoService;
        this.warehouseService = warehouseService;
        this.stockService = stockService;
        this.productClassifyService = productClassifyService;
        this.stockCheckRegisterService = stockCheckRegisterService;
    }

    @Override
    public Page<StockCheckVo> getPage(StockCheckSelectDto dto) {
        IWrapper<StockCheck> wrapper = getWrapper();
        wrapper.like("sc", StockCheckVo::getCreateTime, dto.getKeyword());
        wrapper.eq("sc", StockCheck::getResult, dto.getResult());
        wrapper.orderByDesc("sc", StockCheck::getId);
        Page<StockCheckVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<StockCheckVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        // 根据id赋值操作人名称
        UserUtil.assignmentNickName(page.getRecords(), StockCheckVo::getCreateUser, StockCheckVo::setUserName);

        // 赋值仓库名称
        warehouseService.attributeAssign(records, StockCheck::getWarehouseId, (item, warehouse) -> {
            item.setWarehouseName(warehouse.getName());
        });

        return page;
    }

    @Override
    public StockCheckVo detail(Long id) {
        StockCheckVo stockCheckVo = baseMapper.detail(id);

        List<StockCheckDetailsVo> list = stockCheckVo.getList();
        if (ObjectUtil.isEmpty(list)) {
            return stockCheckVo;
        }
        productInfoService.attributeAssign(list, StockCheckDetails::getProductId, (item, product) -> {
            item.setProductCode(product.getCode());
            item.setProductName(product.getName());
            item.setProductUnit(product.getUnit());
        });
        return stockCheckVo;
    }

    @Override
    @DSTransactional
    public void add(StockCheckDto stockCheckDto) {

        long stockCheckId = IdWorker.getId();
        List<StockCheckDetails> list = stockCheckDto.getList();
        int totalNum = 0, normalNum = 0, anomalyNum = 0;

        for (StockCheckDetails stockCheckDetails : list) {

            totalNum++;

            if (Objects.equals(stockCheckDetails.getQuantity(), stockCheckDetails.getCheckQuantity())) {
                normalNum++;
            } else {
                anomalyNum++;
            }

            stockCheckDetails.setStockCheckId(stockCheckId);
        }

        stockCheckDto.setId(stockCheckId);
        stockCheckDto.setCode(CodeEnum.CHECK_CODE.getCode());
        stockCheckDto.setResult(anomalyNum > 0 ? StatusConstant.YES : StatusConstant.NO);
        stockCheckDto.setTotalNum(totalNum);
        stockCheckDto.setNormalNum(normalNum);
        stockCheckDto.setAnomalyNum(anomalyNum);

        // 保存盘点记录以及明细
        save(stockCheckDto);
        stockCheckDetailsService.saveBatch(list);

        for (StockCheckDetails stockCheckDetails : list) {
            Stock stock = new Stock();
            stock.setProductId(stockCheckDetails.getProductId());
            stock.setQuantity(new BigDecimal(stockCheckDetails.getCheckQuantity()));

            stockService.ModifyInventory(null,5, Collections.singletonList(stock),stockCheckDto.getWarehouseId());
        }
    }

    @Override
    public Page<ProductInfoVo> productPage(ProductInfoSelectDto dto) {
        Assert.notEmpty(dto.getWarehouseId(), "仓库id不能为空！");

        Page<ProductInfoVo> page = productInfoService.getPage(dto);
        List<ProductInfoVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        //赋值库存信息
        List<Long> pIds = records.stream().map(ProductInfo::getId).collect(Collectors.toList());
        Map<Long, Stock> stockMap = stockService.mapKEntity(Stock::getProductId, q -> q
                .in(Stock::getProductId, pIds)
                .eq(Stock::getWarehouseId, dto.getWarehouseId())
        );
        for (ProductInfoVo record : records) {
            //赋值库存
            record.setStockQuantity(BigDecimal.ZERO);

            Stock stock = stockMap.get(record.getId());
            if (ObjectUtil.isNotEmpty(stock)) {
                record.setStockQuantity(stock.getQuantity());
            }
        }
        return page;
    }

    @Override
    public List<IssueImportExcelBo> excelImport(MultipartFile file, Long warehouseId) {
        List<IssueImportExcelBo> issueImportExcelBos = productInfoService.issueImportExcel(file);

        List<Long> pIds = issueImportExcelBos.stream().map(IssueImportExcelBo::getProductId).collect(Collectors.toList());
        Map<Long, BigDecimal> stockMap = stockService.mapKV(Stock::getProductId, Stock::getQuantity, q -> q.in(Stock::getProductId, pIds).eq(Stock::getWarehouseId, warehouseId));
        //赋值库存信息
        for (IssueImportExcelBo issueImportExcelBo : issueImportExcelBos) {
            BigDecimal bigDecimal = stockMap.getOrDefault(issueImportExcelBo.getProductId(), BigDecimal.ZERO);
            issueImportExcelBo.setStockCount(bigDecimal);
        }

        return issueImportExcelBos;
    }

    @Override
    public Page<StockVo> stockProductPage(StockSelectDto dto) {
        IWrapper<Stock> wrapper = IWrapper.getWrapper();

        //仓库过滤
        wrapper.eq("s", Stock::getWarehouseId, dto.getWarehouseId());

        //关键字
        String keyword = dto.getKeyword();
        if (ObjectUtil.isNotEmpty(keyword)) {
            //根据 库存数量 产品自定义编码 产品名称 过滤
            List<Long> productIds = productInfoService.listObject(ProductInfo::getId,
                    q -> q.like(ProductInfo::getCustomCode, keyword).or().like(ProductInfo::getName, keyword));
            wrapper.and(q -> q.like("s", Stock::getQuantity, keyword).or().in("s", Stock::getProductId, productIds));
        }

        //已登记过滤
        Integer isCheckRegister = dto.getIsCheckRegister();
        if (ObjectUtil.isNotEmpty(isCheckRegister)) {
            List<Long> sIds = stockCheckRegisterService.listObject(StockCheckRegister::getStockId, q -> q.orderByAsc(StockCheckRegister::getId));
            sIds.add(-1L);
            if (ObjectUtil.equals(isCheckRegister, 1)) {
                //过滤已经登记
                wrapper.in("s", Stock::getId, sIds);
            } else {
                //过滤没有登记
                wrapper.notIn("s", Stock::getId, sIds);
            }
        }

        wrapper.orderByAsc("s", Stock::getId);
        wrapper.orderByDesc("pi.name");
        Page<StockVo> page = baseMapper.stockProductPage(dto.getPage(), wrapper);
        List<StockVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }

        List<Long> ids = records.stream().map(Stock::getId).collect(Collectors.toList());

        //赋值产品信息
        productInfoService.attributeAssign(records, StockVo::getProductId, (item, productInfo) -> {
            item.setProductCode(productInfo.getCode());
            item.setProductUnit(productInfo.getUnit());
            item.setProductType(productInfo.getType());
            item.setProductName(productInfo.getName());
            item.setProductSpec(productInfo.getSpec());
            item.setProductClassifyId(productInfo.getProductClassifyId());
            item.setProductDefinition(productInfo.getDefinition());
            item.setProductCustomCode(productInfo.getCustomCode());
        });
        //赋值产品分类
        productClassifyService.attributeAssign(records, StockVo::getProductClassifyId, (item, productClassify) -> {
            item.setProductClassifyName(productClassify.getName());
        });

        //赋值仓库名称
        warehouseService.attributeAssign(records, StockVo::getWarehouseId, (item, warehouse) -> {
            item.setWarehouseName(warehouse.getName());
        });

        //赋值盘点登记记录列表
        List<StockCheckRegisterVo> stockCheckRegisterList = stockCheckRegisterService.getList(IWrapper.<StockCheckRegister>getWrapper()
                .in(StockCheckRegister::getStockId, ids)
        );
        Map<Long, List<StockCheckRegisterVo>> checkRegisterMap = stockCheckRegisterList.stream().collect(Collectors.groupingBy(StockCheckRegister::getStockId));
        for (StockVo stockVo : records) {
            stockVo.setStockCheckRegisterList(checkRegisterMap.getOrDefault(stockVo.getId(), new ArrayList<>()));
        }
        return page;
    }

    @DSTransactional
    @Override
    public synchronized void submitAll() {
        List<StockCheckRegister> list = stockCheckRegisterService.list();
        Assert.notEmpty(list, "无盘点登记信息，盘点提交失败！");
        Map<Long, List<StockCheckRegister>> collect = list.stream().collect(Collectors.groupingBy(StockCheckRegister::getWarehouseId));
        for (Map.Entry<Long, List<StockCheckRegister>> entry : collect.entrySet()) {

            //生成盘点明细
            List<StockCheckDetails> stockCheckDetailsList = new ArrayList<>();

            //根据库存id分组
            Map<Long, List<StockCheckRegister>> collect1 = entry.getValue().stream().collect(Collectors.groupingBy(StockCheckRegister::getStockId));
            for (Map.Entry<Long, List<StockCheckRegister>> entry1 : collect1.entrySet()) {
                List<StockCheckRegister> value = entry1.getValue();
                Long productId = value.get(0).getProductId();
                Long checkQuantity = value.stream().map(StockCheckRegister::getCheckQuantity).reduce(Long::sum).orElse(0L);

                //获取数据库库存
                Stock baseStock = stockService.getById(entry1.getKey());

                StockCheckDetails stockCheckDetails = new StockCheckDetails();
                stockCheckDetails.setProductId(productId);
                stockCheckDetails.setQuantity(baseStock.getQuantity().longValue());
                stockCheckDetails.setCheckQuantity(checkQuantity);
                stockCheckDetailsList.add(stockCheckDetails);
            }

            //提交盘点
            StockCheckDto stockCheckDto = new StockCheckDto();
            stockCheckDto.setWarehouseId(entry.getKey());
            stockCheckDto.setList(stockCheckDetailsList);
            add(stockCheckDto);
        }
        //删除盘点记录信息
        stockCheckRegisterService.removeBatchByIds(list);
    }

}
